// GeoProver | Version 1.3a | Jan 20 2003
// Author: H.-G. Graebe, Univ. Leipzig, Germany
// http://www.informatik.uni-leipzig.de/~graebe

// GeoProver interface 

geoprover:=newDomain("geoprover"):
geoprover::info:="The package GeoProver is a small package for mechanized\
 \n(plane) geometry manipulations with non-degeneracy tracing. \n":

geoprover::interface:={
hold(Point),
hold(altitude),
hold(angle_sum),
hold(centroid),
hold(circle_center),
hold(circle_slider),
hold(circle_sqradius),
hold(circumcenter),
hold(csym_point),
hold(eq_angle),
hold(eq_dist),
hold(fixedpoint),
hold(intersection_point),
hold(is_cc_tangent),
hold(is_cl_tangent),
hold(is_collinear),
hold(is_concurrent),
hold(is_concyclic),
hold(is_equal),
hold(is_orthogonal),
hold(is_parallel),
hold(l2_angle),
hold(line_slider),
hold(median),
hold(midpoint),
hold(on_bisector),
hold(on_circle),
hold(on_line),
hold(ortho_line),
hold(orthocenter),
hold(other_cc_point),
hold(other_cl_point),
hold(other_incenter),
hold(p3_angle),
hold(p3_circle),
hold(p9_center),
hold(p9_circle),
hold(p_bisector),
hold(pappus_line),
hold(par_line),
hold(par_point),
hold(pc_circle),
hold(pedalpoint),
hold(pp_line),
hold(radical_axis),
hold(rotate),
hold(sqrdist),
hold(sqrdist_pl),
hold(sym_line),
hold(sym_point),
hold(triangle_area),
hold(varpoint),
hold(clear_ndg),
hold(print_ndg),
hold(add_ndg),
hold(geo_plot)}:


#++

Inline GeoProver code Version 1.3a

Data structures:

        Point A  :== [a1,a2]       <=> A=(a1,a2)
        Line  a  :== [a1,a2,a3]    <=> a1*x+a2*y+a3 = 0
	Circle c :== [c0,c1,c2,c3] <=> c0*(x^2+y^2)+c1*x+c2*y+c3 = 0

++#

# a hook for a domain concept #

geoprover::DOM_GEO:=DOM_LIST:         # superdomain #
geoprover::DOM_POINT:=DOM_LIST:
geoprover::DOM_LINE:=DOM_LIST:
geoprover::DOM_CIRCLE:=DOM_LIST:

// ================================================================
// ============= Handling non degeneracy conditions =============== 
// ================================================================

geoprover::clear_ndg:=
proc() begin geoprover::ndg_list:=[]: end_proc:

geoprover::clear_ndg(): // Initialization

geoprover::print_ndg:=
proc() begin print(geoprover::ndg_list) end_proc:

geoprover::add_ndg:=proc(d) 
begin 
  if contains(geoprover::ndg_list,d)=0 then
    geoprover::ndg_list:=append(geoprover::ndg_list,d) 
  end_if 
end_proc:

geoprover::reduce_coords:=proc(u)
// Divide out the content of homogeneous coordinates.
  local p,l,g,i;
begin
  p:=op(u,1); l:=denom(p); g:=numer(p);
  for i from 2 to nops(u) do 
	p:=op(u,i); l:=lcm(l,denom(p)); g:=gcd(g,numer(p)) end_for;
  if g=0 then error("gcd=0!") end_if;
  geoprover::add_ndg(g);
  map(u,x->normal(x*l/g)); 
end_proc:

// ====================================================================
// ================= elementary geometric constructions =============== 
// ====================================================================

// Generators:

geoprover::is_equal:=proc(a,b) begin normal(a-b) end_proc:

geoprover::Point:=proc(a,b):geoprover::DOM_POINT
begin [normal(a),normal(b)] end_proc:

geoprover::Line:=proc(a,b,c):geoprover::DOM_LINE
begin geoprover::reduce_coords([normal(a),normal(b),normal(c)]) 
end_proc:

geoprover::par_point:= 
proc(a:geoprover::DOM_POINT, b:geoprover::DOM_POINT,
      c:geoprover::DOM_POINT):geoprover::DOM_POINT 
// Point D that makes ABCD a parallelogram.
begin geoprover::Point(c[1]-b[1]+a[1], c[2]-b[2]+a[2]) 
end_proc:

geoprover::pp_line:=
proc(a:geoprover::DOM_POINT,b:geoprover::DOM_POINT):geoprover::DOM_LINE
// the line through A and B 
begin 
  geoprover::Line(op(b,2)-op(a,2),op(a,1)-op(b,1),
  op(a,2)*op(b,1)-op(a,1)*op(b,2)) 
end_proc:

geoprover::intersection_point:=
proc(a:geoprover::DOM_LINE,b:geoprover::DOM_LINE):geoprover::DOM_POINT
// The intersection point of the lines a,b.
   local d,d1,d2;
begin
   d:=normal(op(a,1)*op(b,2)-op(b,1)*op(a,2));
   d1:=op(a,3)*op(b,2)-op(b,3)*op(a,2);
   d2:=op(a,1)*op(b,3)-op(b,1)*op(a,3);
   if d=0 then error("Lines are parallel") end_if;
   geoprover::add_ndg(numer(d));
   geoprover::Point(-d1/d,-d2/d);
end_proc:

geoprover::ortho_line:=
proc(p:geoprover::DOM_POINT,a:geoprover::DOM_LINE):geoprover::DOM_LINE
// The ortho_line from P onto the line a.
  local u,v;
begin
  u:=op(a,1); v:=op(a,2);
  geoprover::Line(v,-u,u*op(p,2)-v*op(p,1));
end_proc:

geoprover::par_line:=
proc(P:geoprover::DOM_POINT,a:geoprover::DOM_LINE):geoprover::DOM_LINE
// The parallel line to line a through P.
begin
  geoprover::Line(op(a,1),op(a,2),-(op(a,1)*op(P,1)+op(a,2) *op(P,2))) 
end_proc:

geoprover::varpoint:=
proc(b:geoprover::DOM_POINT,a:geoprover::DOM_POINT,l):geoprover::DOM_POINT
// The point D=l*A+(1-l)*B.
begin
  geoprover::Point(l*op(a,1)+(1-l)*op(b,1),l*op(a,2)+(1-l)*op(b,2)) 
end_proc:

geoprover::line_slider:=proc(a:geoprover::DOM_LINE,u):geoprover::DOM_POINT
// Choose a point on the line a using parameter u.
  local p,d;
begin
  if op(a,2)=0 then 
	if op(a,1)=0 then error("Line is degenerate") end_if;
	p:=geoprover::Point(-op(a,3)/op(a,1),u); d:=op(a,1); 
  else 	
	p:=geoprover::Point(u,-(op(a,3)+op(a,1)*u)/op(a,2));
	d:=op(a,2); 
  end_if;
  geoprover::add_ndg(numer(d));
  p;
end_proc:

geoprover::sqrdist:=proc(a:geoprover::DOM_POINT,b:geoprover::DOM_POINT)
// The square of the distance between the points A and B.
begin  
  normal((op(b,1)-op(a,1))^2+(op(b,2)-op(a,2))^2) 
end_proc:

geoprover::eq_dist:=proc(a:geoprover::DOM_POINT,b:geoprover::DOM_POINT,
	c:geoprover::DOM_POINT,d:geoprover::DOM_POINT)
// distances AB and CD are equal.
begin  
  normal(geoprover::sqrdist(a,b)-geoprover::sqrdist(c,d)) 
end_proc:

// =================================================================
// ================= elementary geometric properties ===============
// =================================================================

geoprover::is_collinear:=
proc(a:geoprover::DOM_POINT,b:geoprover::DOM_POINT,c:geoprover::DOM_POINT)
// A,B,C are on a common line.
begin
  normal(linalg::det(Dom::Matrix()(
		   [[op(a,1),op(a,2),1],
		    [op(b,1),op(b,2),1],
		    [op(c,1),op(c,2),1]]))) 
end_proc:

geoprover::is_concurrent:=
proc(a:geoprover::DOM_LINE,b:geoprover::DOM_LINE,c:geoprover::DOM_LINE)
// Lines a,b,c have a common point.
begin
  normal(linalg::det(Dom::Matrix()(
	[[op(a,1),op(a,2),op(a,3)],
	 [op(b,1),op(b,2),op(b,3)],
	 [op(c,1),op(c,2),op(c,3)]]))) 
end_proc:

geoprover::is_parallel:=proc(a:geoprover::DOM_LINE,b:geoprover::DOM_LINE)
// 0 <=> the lines a,b are parallel.
begin  
  normal(op(a,1)*op(b,2)-op(b,1)*op(a,2)) 
end_proc:

geoprover::is_orthogonal:=proc(a:geoprover::DOM_LINE,b:geoprover::DOM_LINE)
// 0 <=> the lines a,b are orthogonal.
begin
  normal(op(a,1)*op(b,1)+op(a,2)*op(b,2)) 
end_proc:

geoprover::on_line:=proc(p:geoprover::DOM_POINT,a:geoprover::DOM_LINE)
// Substitute point P into the line a.
begin  
  normal(op(p,1)*op(a,1)+op(p,2)*op(a,2)+op(a,3)) 
end_proc:

// ================================================================
// ================= nonlinear geometric objects ==================
// ================================================================

// ====================== angles ==================================

geoprover::l2_angle:=proc(a:geoprover::DOM_LINE,b:geoprover::DOM_LINE)
// tan of the angle between the lines a and b.
  local d; 
begin
  d:=normal(op(a,1)*op(b,1)+op(a,2)*op(b,2)); 
  if d=0 then error("tan is indefinite") end_if;
  geoprover::add_ndg(numer(d));
  normal((op(a,2)*op(b,1)-op(b,2)*op(a,1))/d);
end_proc:

geoprover::p3_angle:=
proc(A:geoprover::DOM_POINT,B:geoprover::DOM_POINT,C:geoprover::DOM_POINT)
// tan of the angle between the lines BA and BC
begin  
  geoprover::l2_angle(geoprover::pp_line(B,A),geoprover::pp_line(B,C)) 
end_proc:

geoprover::angle_sum:=proc(a,b)
// a=tan(\alpha), b=tan(\beta). Returns tan(\alpha+\beta)
  local d;
begin 
  d:=normal(1-a*b); geoprover::add_ndg(numer(d)); 
  if d=0 then error("tan is indefinite") end_if;
  normal((a+b)/d);
end_proc:

geoprover::eq_angle:=
proc(a:geoprover::DOM_POINT,b:geoprover::DOM_POINT,c:geoprover::DOM_POINT,
	d:geoprover::DOM_POINT,e:geoprover::DOM_POINT,f:geoprover::DOM_POINT)
// Angles ABC and DEF are equal.
begin normal(geoprover::p3_angle(a,b,c)-geoprover::p3_angle(d,e,f)) end_proc:

geoprover::on_bisector:=proc(P:geoprover::DOM_POINT,A:geoprover::DOM_POINT,
	B:geoprover::DOM_POINT,C:geoprover::DOM_POINT)
// P is a point on the bisector of the angle ABC.
// Returns num(u)*den(v)-num(v)*den(u) with
// u:=angle(pp_line(A,B),pp_line(P,B))
// v:=angle(pp_line(P,B),pp_line(C,B))
  local a1,a2,b1,b2,c1,c2,p1,p2;
begin
  a1:=op(A,1); a2:=op(A,2);
  b1:=op(B,1); b2:=op(B,2);
  c1:=op(C,1); c2:=op(C,2);
  p1:=op(P,1); p2:=op(P,2);

  normal( normal( - a1*b2 + a1*p2 + a2*b1 - a2*p1 - b1*p2 +
   b2*p1)* normal( b1^2 - b1*c1 - b1*p1 + b2^2 - b2*c2 - b2*p2 +
   c1*p1 + c2*p2) - normal(a1*b1 - a1*p1 + a2*b2 - a2*p2 - b1^2 +
   b1*p1 - b2^2 + b2*p2) * normal(b1*c2 - b1*p2 - b2*c1 + b2*p1 +
   c1*p2 - c2*p1));

end_proc:

geoprover::rotate:= 
proc(C:geoprover::DOM_POINT, A:geoprover::DOM_POINT, angle)
begin
  geoprover::Point(
    C[1]+(A[1]-C[1])*cos(angle*PI)-(A[2]-C[2])*sin(angle*PI),
    C[2]+(A[1]-C[1])*sin(angle*PI)+(A[2]-C[2])*cos(angle*PI)) 
end_proc:

// ================ symmetric lines and points ====================

geoprover::sym_line:=
proc(a:geoprover::DOM_LINE,l:geoprover::DOM_LINE):geoprover::DOM_LINE
// The line symmetric to a wrt. the line l.
  local a1,a2,a3,l1,l2,l3,u;
begin
  a1:=op(a,1); a2:=op(a,2); a3:=op(a,3); 
  l1:=op(l,1); l2:=op(l,2); l3:=op(l,3); 
  u:=normal(l1^2 - l2^2);
  geoprover::Line(- a1*u - 2*a2*l1*l2, - 2*a1*l1*l2 + a2*u, 
		- 2*(a1*l1 + a2*l2)*l3 + a3*(l1^2 + l2^2));
end_proc:

// ================================================================
// ====================== circles =================================
// ================================================================

geoprover::circle_slider:=
proc(M:geoprover::DOM_POINT,A:geoprover::DOM_POINT,u):geoprover::DOM_POINT
  local d,a1,a2,m1,m2; 
begin a1:=op(A,1); a2:=op(A,2); m1:=op(M,1); m2:=op(M,2);
  d:=normal(u^2+1); geoprover::add_ndg(numer(d));
  geoprover::Point((a1*(u^2-1) + 2*m1 + 2*(m2-a2)*u)/d, 
	 (a2 + 2*(m1-a1)*u + (2*m2-a2)*u^2)/d);
end_proc:

geoprover::Circle:=proc(c1,c2,c3,c4):geoprover::DOM_CIRCLE
begin
  geoprover::reduce_coords([normal(c1),normal(c2),normal(c3),normal(c4)])
end_proc:

geoprover::pc_circle:=
proc(M:geoprover::DOM_POINT,A:geoprover::DOM_POINT):geoprover::DOM_CIRCLE
// Circle from center M and circumfere point A.
begin  
 geoprover::Circle(1, -2*op(M,1), -2*op(M,2),
    op(A,1)*(2*op(M,1)-op(A,1)) + op(A,2)*(2*op(M,2)-op(A,2)))
end_proc:

geoprover::circle_center:=proc(c:geoprover::DOM_CIRCLE):geoprover::DOM_POINT
// The center of the circle c.
begin  
  geoprover::add_ndg(numer(op(c,1)));
  geoprover::Point(-op(c,2)/2/op(c,1) ,-op(c,3)/(2*op(c,1)));
end_proc:

geoprover::circle_sqradius:=proc(c:geoprover::DOM_CIRCLE)
// The squared radius of the circle c.
begin
  geoprover::add_ndg(numer(op(c,1)));
  normal(((op(c,2)^2+op(c,3)^2) - 4*op(c,4)*op(c,1)) / 
	(2*op(c,1))^2);
end_proc:

geoprover::p3_circle:=proc(A:geoprover::DOM_POINT,B:geoprover::DOM_POINT,
	C:geoprover::DOM_POINT):geoprover::DOM_CIRCLE
// The circle through three given points
  local a1,a2,a3,b1,b2,b3,c1,c2,c3;
begin  
  a1:=op(A,1); a2:=op(A,2); a3:=normal(a1^2+a2^2);
  b1:=op(B,1); b2:=op(B,2); b3:=normal(b1^2+b2^2);
  c1:=op(C,1); c2:=op(C,2); c3:=normal(c1^2+c2^2);
  geoprover::Circle(a1*(b2-c2) + (a2-b2)*c1 + b1*(c2-a2),
    a3*(c2-b2) + (a2-c2)*b3 + (b2-a2)*c3,
    a3*(b1-c1) + (c1-a1)*b3 + (a1-b1)*c3,
    a3*(b2*c1-b1*c2) + (a1*c2-a2*c1)*b3 + (a2*b1-a1*b2)*c3)
end_proc:

geoprover::on_circle:=proc(P:geoprover::DOM_POINT,c:geoprover::DOM_CIRCLE)
// Checks if P is on circle c  
local p1,p2; 
begin
  p1:=op(P,1); p2:=op(P,2);
  normal(op(c,1)*(p1^2+p2^2)+op(c,2)*p1+op(c,3)*p2+op(c,4));
end_proc:

// ============= intersecting with circles ========================

geoprover::other_cl_point:=
proc(P:geoprover::DOM_POINT,c:geoprover::DOM_CIRCLE,
	l:geoprover::DOM_LINE):geoprover::DOM_POINT
// circle c and line l intersect at P. The procedure returns their
// second intersection point.
  local c1,c2,c3,l1,l2,d,d1,p1,p2;
begin
  if geoprover::on_line(P,l) <> 0 then error( "Point not on the line")
  elif geoprover::on_circle(P,c) <> 0 then 
	error( "Point not on the circle")
  else 
     c1:=op(c,1); c2:=op(c,2); c3:=op(c,3); 
     l1:=op(l,1); l2:=op(l,2); p1:=op(P,1); p2:=op(P,2);
     d:=normal(c1*(l1^2 + l2^2));
	if d=0 then error("circle or line degenerate") end_if;
     geoprover::add_ndg(numer(d)); 
     d1:=normal(c1*(l1^2-l2^2));
     geoprover::Point((d1*p1+((2*c1*p2 + c3)*l1-c2*l2)*l2)/d, 
	  (- d1*p2+((2*c1*p1 + c2)*l2-c3*l1)*l1)/d);
  end_if
end_proc:

geoprover::radical_axis:=proc(c1:geoprover::DOM_CIRCLE,
	c2:geoprover::DOM_CIRCLE):geoprover::DOM_LINE
// Radical axis of the circles c1 and c2, i.e. the line through the
// intersection points of the two circles if they intersect. 
  local i;
begin 
  geoprover::Line(normal(op(c1,1)*op(c2,i)-op(c1,i)*op(c2,1))$i=2..4 ); 
end_proc:

geoprover::other_cc_point:=
proc(P:geoprover::DOM_POINT,c1:geoprover::DOM_CIRCLE,
	c2:geoprover::DOM_CIRCLE):geoprover::DOM_POINT
// Circles c1 and c2 intersect at P. The procedure returns their
// second intersection point.
begin 
  geoprover::other_cl_point(P,c1,geoprover::radical_axis(c1,c2));
end_proc:

geoprover::is_cl_tangent:=
proc(c:geoprover::DOM_CIRCLE,l:geoprover::DOM_LINE)
// Line l is tangent to the circle c.
  local c1,c2,c3,c4,l1,l2,l3;
begin
  c1:=op(c,1); c2:=op(c,2); c3:=op(c,3); c4:=op(c,4); 
  l1:=op(l,1); l2:=op(l,2); l3:=op(l,3); 
  normal(4*c1*( (- c1*l3 + c2*l1 + c3*l2)*l3 - c4*(l1^2 + l2^2))  
	+ (c2*l2 - c3*l1)^2); 
end_proc:

geoprover::is_cc_tangent:=
proc(c:geoprover::DOM_CIRCLE,d:geoprover::DOM_CIRCLE)
// Two circles c,d are tangent.
  local c1,c2,c3,c4,d1,d2,d3,d4;
begin
  c1:=op(c,1); c2:=op(c,2); c3:=op(c,3); c4:=op(c,4); 
  d1:=op(d,1); d2:=op(d,2); d3:=op(d,3); d4:=op(d,4); 
  normal(4*(c1*d4-c4*d1)^2 - (c2*d3-c3*d2)^2 
	- 4*(c1*d4+c4*d1)*(c2*d2+c3*d3) + 4*c1*c4*(d2^2 + d3^2) 
	+ 4*(c2^2 + c3^2)*d1*d4) ;
end_proc:


// =========GeoProver's undocumented Plot addendum ============

#

geo_plot(Options ..., Scenes ...)

Options == usual MuPAD plot options.

Scene ==  [List of GeoProver objects, range <,Color>]

#

geoprover::geo_plot:=proc()
  local sceneopt, j, obj_seq;
begin
  sceneopt:= Axes = None, Scaling = Constrained, 
	     select(args(),testtype,"_equal");
  obj_seq:=select(args(),testtype,DOM_LIST);
  obj_seq:=map(obj_seq,op@geoprover::plot1@op);
  plot::Scene(sceneopt, obj_seq) 
end_proc:

geoprover::plot1:=proc(Objects:DOM_LIST,
        Range:"_equal",color=(Color=RGB::Black))
begin
  map(Objects,geoprover::plot2,Range,color);
end_proc:

geoprover::plot2:=proc(geo_obj:DOM_LIST,range,color)
begin
  if nops(geo_obj)=2 then
	geoprover::plot_point(geo_obj,color);
  elif nops(geo_obj)=3 then
	geoprover::plot_line(geo_obj,range,color);
  else
	if op(geo_obj,1)=0 then
		geoprover::plot_line([op(geo_obj,2..4)],range,color);
	else
		geoprover::plot_circle(geo_obj,color);
	end_if;
  end_if;
end_proc:

geoprover::plot_point:=proc(o:geoprover::DOM_POINT,c)
begin plot::Point(op(o),c) end_proc:

geoprover::plot_line:=proc(l:geoprover::DOM_LINE,r,c)
  local rhs;
begin
  rhs:=op(r,2);
  if l[2]=0 then
     plot::line( plot::Point(-l[3]/l[1],op(rhs,1)),
		 plot::Point(-l[3]/l[1],op(rhs,2)), c);
  else plot::Function2d(-(l[1]*x+l[3])/l[2],x=rhs,y=rhs,c);
  end_if;
end_proc:

geoprover::plot_circle:=proc(p:geoprover::DOM_LINE,c)
  local M,r,N,pi_N;
begin 
  M:=geoprover::circle_center(p);
  r:=geoprover::circle_sqradius(p);
  if r<0 then error("Circle has negative sqradius") end_if; 
  r:=float(sqrt(r)); N:=50; pi_N:=float(PI/N);
  plot::Pointlist(
      plot::Point(M[1]+r*cos(i*pi_N),M[2]+r*sin(i*pi_N),c) $ i=-N..N,
      DrawMode = Connected, PointStyle=Circles, PointWidth=1, c);
end_proc:


// GeoProver code generated from database 

geoprover::altitude:=
proc(_A:geoprover::DOM_POINT,_B:geoprover::DOM_POINT,_C:geoprover::DOM_POINT):geoprover::DOM_LINE
begin
geoprover::ortho_line(_A,geoprover::pp_line(_B,_C))
end_proc:

geoprover::centroid:=
proc(_A:geoprover::DOM_POINT,_B:geoprover::DOM_POINT,_C:geoprover::DOM_POINT):geoprover::DOM_POINT
begin
geoprover::intersection_point(geoprover::median(_A,_B,_C),geoprover::median(_B,_C,_A))
end_proc:

geoprover::circumcenter:=
proc(_A:geoprover::DOM_POINT,_B:geoprover::DOM_POINT,_C:geoprover::DOM_POINT):geoprover::DOM_POINT
begin
geoprover::intersection_point(geoprover::p_bisector(_A,_B), geoprover::p_bisector(_B,_C))
end_proc:

geoprover::csym_point:=
proc(_P:geoprover::DOM_POINT,_Q:geoprover::DOM_POINT):geoprover::DOM_POINT
begin
geoprover::varpoint(_Q,_P,-1)
end_proc:

geoprover::fixedpoint:=
proc(_A:geoprover::DOM_POINT,_B:geoprover::DOM_POINT,_u):geoprover::DOM_POINT
begin
geoprover::varpoint(_A,_B,_u)
end_proc:

geoprover::is_concyclic:=
proc(_A:geoprover::DOM_POINT,_B:geoprover::DOM_POINT,_C:geoprover::DOM_POINT,_D:geoprover::DOM_POINT)
begin
geoprover::on_circle(_D,geoprover::p3_circle(_A,_B,_C))
end_proc:

geoprover::median:=
proc(_A:geoprover::DOM_POINT,_B:geoprover::DOM_POINT,_C:geoprover::DOM_POINT):geoprover::DOM_LINE
begin
geoprover::pp_line(_A,geoprover::midpoint(_B,_C))
end_proc:

geoprover::midpoint:=
proc(_A:geoprover::DOM_POINT,_B:geoprover::DOM_POINT):geoprover::DOM_POINT
begin
geoprover::fixedpoint(_A,_B,1/2)
end_proc:

geoprover::orthocenter:=
proc(_A:geoprover::DOM_POINT,_B:geoprover::DOM_POINT,_C:geoprover::DOM_POINT):geoprover::DOM_POINT
begin
geoprover::intersection_point(geoprover::altitude(_A,_B,_C),geoprover::altitude(_B,_C,_A))
end_proc:

geoprover::other_incenter:=
proc(_M:geoprover::DOM_POINT,_A:geoprover::DOM_POINT,_B:geoprover::DOM_POINT):geoprover::DOM_POINT
begin
geoprover::intersection_point(geoprover::ortho_line(_A,geoprover::pp_line(_M,_A)),
geoprover::ortho_line(_B,geoprover::pp_line(_M,_B)))
end_proc:

geoprover::p3_angle:=
proc(_A:geoprover::DOM_POINT,_B:geoprover::DOM_POINT,_C:geoprover::DOM_POINT)
begin
geoprover::l2_angle(geoprover::pp_line(_B,_A),geoprover::pp_line(_B,_C))
end_proc:

geoprover::p9_center:=
proc(_A:geoprover::DOM_POINT,_B:geoprover::DOM_POINT,_C:geoprover::DOM_POINT):geoprover::DOM_POINT
begin
geoprover::circle_center(geoprover::p9_circle(_A,_B,_C))
end_proc:

geoprover::p9_circle:=
proc(_A:geoprover::DOM_POINT,_B:geoprover::DOM_POINT,_C:geoprover::DOM_POINT):geoprover::DOM_CIRCLE
begin
geoprover::p3_circle(geoprover::midpoint(_A,_B),geoprover::midpoint(_A,_C),geoprover::midpoint(_B,_C))
end_proc:

geoprover::p_bisector:=
proc(_B:geoprover::DOM_POINT,_C:geoprover::DOM_POINT):geoprover::DOM_LINE
begin
geoprover::ortho_line(geoprover::midpoint(_B,_C),geoprover::pp_line(_B,_C))
end_proc:

geoprover::pappus_line:=
proc(_A:geoprover::DOM_POINT,_B:geoprover::DOM_POINT,_C:geoprover::DOM_POINT,_D:geoprover::DOM_POINT,_E:geoprover::DOM_POINT,_F:geoprover::DOM_POINT):geoprover::DOM_LINE
begin
geoprover::pp_line(geoprover::intersection_point(geoprover::pp_line(_A,_E),geoprover::pp_line(_B,_D)),
	geoprover::intersection_point(geoprover::pp_line(_A,_F),geoprover::pp_line(_C,_D)))
end_proc:

geoprover::pedalpoint:=
proc(_P:geoprover::DOM_POINT,_a:geoprover::DOM_LINE):geoprover::DOM_POINT
begin
geoprover::intersection_point(geoprover::ortho_line(_P,_a),_a)
end_proc:

geoprover::sqrdist_pl:=
proc(_A:geoprover::DOM_POINT,_l:geoprover::DOM_LINE)
begin
geoprover::sqrdist(_A,geoprover::pedalpoint(_A,_l))
end_proc:

geoprover::sym_point:=
proc(_P:geoprover::DOM_POINT,_l:geoprover::DOM_LINE):geoprover::DOM_POINT
begin
geoprover::fixedpoint(_P,geoprover::pedalpoint(_P,_l),2)
end_proc:

geoprover::triangle_area:=
proc(_A:geoprover::DOM_POINT,_B:geoprover::DOM_POINT,_C:geoprover::DOM_POINT)
begin
1/2*geoprover::is_collinear(_A,_B,_C)
end_proc:

// GeoProver code end.
